@@ -6,14 +6,14 @@ import (
 	"os"
 	"path/filepath"
 
+	"github.com/gruntwork-io/terragrunt/internal/component"
 	"github.com/gruntwork-io/terragrunt/internal/report"
 	"github.com/gruntwork-io/terragrunt/options"
 	"github.com/gruntwork-io/terragrunt/tf"
 	"github.com/gruntwork-io/terragrunt/util"
 )
 
-// UnitStatus represents the status of a unit that we are
-// trying to apply or destroy as part of the run --all apply or run --all destroy command
+// UnitStatus represents the status of a unit during execution.
 type UnitStatus int
 
 const (
@@ -22,42 +22,41 @@ const (
 	Finished
 )
 
-// UnitRunner handles the logic for running a single unit.
+// UnitRunner handles the logic for running a single component.Unit.
 type UnitRunner struct {
 	Err    error
-	Unit   *Unit
+	Unit   *component.Unit
 	Status UnitStatus
 }
 
-var outputLocks = util.NewKeyLocks()
-
-func NewUnitRunner(unit *Unit) *UnitRunner {
+// NewUnitRunner creates a UnitRunner from a component.Unit.
+func NewUnitRunner(unit *component.Unit) *UnitRunner {
 	return &UnitRunner{
 		Unit:   unit,
 		Status: Waiting,
 	}
 }
 
-func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.TerragruntOptions, r *report.Report) error {
-	runner.Unit.Logger.Debugf("Running %s", runner.Unit.Path)
+// Deprecated: use NewUnitRunner
+func NewUnitRunnerFromComponent(unit *component.Unit) *UnitRunner {
+	return NewUnitRunner(unit)
+}
 
-	opts.Writer = NewUnitWriter(opts.Writer)
+func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.TerragruntOptions, r *report.Report) error {
+	if runner.Unit.Execution == nil || runner.Unit.Execution.Logger == nil {
+		return nil
+	}
+
+	runner.Unit.Execution.Logger.Debugf("Running %s", runner.Unit.Path())
 
 	defer func() {
-		outputLocks.Lock(runner.Unit.Path)
-		defer outputLocks.Unlock(runner.Unit.Path)
-
-		runner.Unit.FlushOutput() //nolint:errcheck
+		// Flush buffered output for this unit, if the writer supports it.
+		_ = component.FlushOutput(runner.Unit)
 	}()
 
 	// Only create report entries if report is not nil
 	if r != nil {
-		// Ensure path is absolute and normalized for reporting
-		unitPath, err := EnsureAbsolutePath(runner.Unit.Path)
-		if err != nil {
-			return err
-		}
-
+		unitPath := runner.Unit.AbsolutePath()
 		unitPath = util.CleanPath(unitPath)
 
 		run, err := report.NewRun(unitPath)
@@ -77,7 +76,7 @@ func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.Terra
 
 	ctx = tf.ContextWithDetailedExitCode(ctx, &unitExitCode)
 
-	runErr := opts.RunTerragrunt(ctx, runner.Unit.Logger, opts, r)
+	runErr := opts.RunTerragrunt(ctx, runner.Unit.Execution.Logger, opts, r)
 
 	// Only merge the final unit exit code when the unit run completed without error
 	// and the exit code isn't stuck at 1 from a prior retry attempt.
@@ -87,8 +86,7 @@ func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.Terra
 
 	// End the run with appropriate result (only if report is not nil)
 	if r != nil {
-		// Get the unit path (already computed above)
-		unitPath := runner.Unit.AbsolutePath(runner.Unit.Logger)
+		unitPath := runner.Unit.AbsolutePath()
 		unitPath = util.CleanPath(unitPath)
 
 		if runErr != nil {
@@ -98,11 +96,11 @@ func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.Terra
 				report.WithReason(report.ReasonRunError),
 				report.WithCauseRunError(runErr.Error()),
 			); endErr != nil {
-				runner.Unit.Logger.Errorf("Error ending run for unit %s: %v", unitPath, endErr)
+				runner.Unit.Execution.Logger.Errorf("Error ending run for unit %s: %v", unitPath, endErr)
 			}
 		} else {
 			if endErr := r.EndRun(unitPath, report.WithResult(report.ResultSucceeded)); endErr != nil {
-				runner.Unit.Logger.Errorf("Error ending run for unit %s: %v", unitPath, endErr)
+				runner.Unit.Execution.Logger.Errorf("Error ending run for unit %s: %v", unitPath, endErr)
 			}
 		}
 	}
@@ -110,22 +108,29 @@ func (runner *UnitRunner) runTerragrunt(ctx context.Context, opts *options.Terra
 	return runErr
 }
 
-// Run a unit right now by executing the runTerragrunt command of its TerragruntOptions field.
+// Run executes a component.Unit right now.
 func (runner *UnitRunner) Run(ctx context.Context, opts *options.TerragruntOptions, r *report.Report) error {
 	runner.Status = Running
 
-	if runner.Unit.AssumeAlreadyApplied {
-		runner.Unit.Logger.Debugf("Assuming unit %s has already been applied and skipping it", runner.Unit.Path)
+	if runner.Unit.Execution == nil {
 		return nil
 	}
 
-	if err := runner.runTerragrunt(ctx, runner.Unit.TerragruntOptions, r); err != nil {
+	if runner.Unit.Execution.AssumeAlreadyApplied {
+		if runner.Unit.Execution.Logger != nil {
+			runner.Unit.Execution.Logger.Debugf("Assuming unit %s has already been applied and skipping it", runner.Unit.Path())
+		}
+
+		return nil
+	}
+
+	if err := runner.runTerragrunt(ctx, runner.Unit.Execution.TerragruntOptions, r); err != nil {
 		return err
 	}
 
 	// convert terragrunt output to json
-	if runner.Unit.OutputJSONFile(runner.Unit.Logger, runner.Unit.TerragruntOptions) != "" {
-		l, jsonOptions, err := runner.Unit.TerragruntOptions.CloneWithConfigPath(runner.Unit.Logger, runner.Unit.TerragruntOptions.TerragruntConfigPath)
+	if runner.Unit.OutputJSONFile(runner.Unit.Execution.TerragruntOptions) != "" {
+		l, jsonOptions, err := runner.Unit.Execution.TerragruntOptions.CloneWithConfigPath(runner.Unit.Execution.Logger, runner.Unit.Execution.TerragruntOptions.TerragruntConfigPath)
 		if err != nil {
 			return err
 		}
@@ -135,18 +140,16 @@ func (runner *UnitRunner) Run(ctx context.Context, opts *options.TerragruntOptio
 		jsonOptions.JSONLogFormat = false
 		jsonOptions.Writer = &stdout
 		jsonOptions.TerraformCommand = tf.CommandNameShow
-		jsonOptions.TerraformCliArgs = []string{tf.CommandNameShow, "-json", runner.Unit.PlanFile(l, opts)}
+		jsonOptions.TerraformCliArgs = []string{tf.CommandNameShow, "-json", runner.Unit.PlanFile(opts)}
 
-		// Use an ad-hoc report to avoid polluting the main report with entries
-		// for the cache directory, while still satisfying RunTerragrunt's
-		// expectation for a non-nil report parameter.
+		// Use an ad-hoc report to avoid polluting the main report
 		adhocReport := report.NewReport()
 		if err := jsonOptions.RunTerragrunt(ctx, l, jsonOptions, adhocReport); err != nil {
 			return err
 		}
 
 		// save the json output to the file plan file
-		outputFile := runner.Unit.OutputJSONFile(l, opts)
+		outputFile := runner.Unit.OutputJSONFile(opts)
 		jsonDir := filepath.Dir(outputFile)
 
 		if err := os.MkdirAll(jsonDir, os.ModePerm); err != nil {
